using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Windows.Forms;

using Syncfusion.Diagnostics;
using Syncfusion.Drawing;
using Syncfusion.Windows.Forms;
using Syncfusion.Windows.Forms.Grid;

namespace ExcelMarker
{

	public class ExcelMarkerMouseController : IMouseController
	{
		GridControlBase owner;
		GridRangeInfo activeRange;
		int lastHitTestCode = GridHitTestContext.None;
		const int HitExcelMarker = 101;
		const int delta = 3;
		private DragWindow dragWindow;
		Cursor cursor1;

		public ExcelMarkerMouseController(GridControlBase owner)
		{
			this.owner = owner;
			this.owner.Model.Options.SelectCellsMouseButtonsMask = MouseButtons.Left;
		}

		public string Name 
		{ 
			get
			{
				return "ExcelMarker"; 
			}
		}

		public Cursor Cursor 
		{ 
			get
			{
				if (cursor1 == null)
				{
					System.IO.Stream stream = GetType().Module.Assembly.GetManifestResourceStream("ExcelMarker.cross.CUR");
					if (stream != null)
						cursor1 = new Cursor(stream);
					else
						cursor1 = Cursors.Cross;

				}
				return cursor1;
			}
		}

		public void MouseHoverEnter()
		{
		}

		public void MouseHover(MouseEventArgs e)
		{
		}
		public void MouseHoverLeave(EventArgs e)
		{
		}

		GridCurrentCell cc;
		public void MouseDown(MouseEventArgs e)
		{
			cc = this.owner.CurrentCell;
			if (e.Button == MouseButtons.Left)
			{
				if (e.Clicks == 1)
				{
					// Start automatic scrolling when user drags mouse out of window
					owner.AutoScrolling = ScrollBars.Both;

					// 
					int rowIndex, colIndex;
					Point point = new Point(e.X, e.Y);
					owner.PointToRowCol(point, out rowIndex, out colIndex);
					activeRange = owner.Selections.Ranges.ActiveRange;

					dragWindow = new DragWindow();
					dragWindow.Opacity = .2f;
					dragWindow.BackColor = SystemColors.Highlight;
					dragWindow.Bounds = GetBounds(activeRange);
					dragWindow.ShowWindowTopMost();
					val = this.owner.CurrentCell.Renderer.ControlValue;
				}
			}
			// else if (e.Clicks == 2)
			// DoubleClick

		}

		Rectangle GetBounds(GridRangeInfo range)
		{
			Rectangle bounds = owner.RangeInfoToRectangle(range, GridRangeOptions.None);
			bounds.Intersect(owner.ClientRectangle);
			bounds = owner.RectangleToScreen(bounds);
			return bounds;
		}


		public void MouseMove(MouseEventArgs e)
		{
			int rowIndex, colIndex;
			Point point = new Point(e.X-delta, e.Y-delta);
			try
			{
				owner.PointToRowCol(point, out rowIndex, out colIndex, 0);
				rowIndex = Math.Max(owner.TopRowIndex, rowIndex);
				colIndex = Math.Max(owner.LeftColIndex, colIndex);
				GridRangeInfo range = GridRangeInfo.Cells(activeRange.Top, activeRange.Left, rowIndex, colIndex);
				dragWindow.Bounds = GetBounds(range);
				dragWindow.ShowWindowTopMost();
			}
			catch
			{
			}
		}

		public void MouseUp(MouseEventArgs e)
		{
			// Stop automatic scrolling 
			owner.AutoScrolling = ScrollBars.None;

			int rowIndex, colIndex;
			Point point = new Point(e.X, e.Y);
			owner.PointToRowCol(point, out rowIndex, out colIndex);

			dragWindow.Visible = false;
			dragWindow = null;

			range = GridRangeInfo.Cells(activeRange.Top, activeRange.Left, rowIndex, colIndex);
			GetDraggedRangesFilled();
/***
			if (e.Button == MouseButtons.Right)
			{
				int top = this.owner.Model.Selections.Ranges.ActiveRange.Top;
				int bottom = this.owner.Model.Selections.Ranges.ActiveRange.Bottom;
				int left = this.owner.Model.Selections.Ranges.ActiveRange.Left;
				int right = this.owner.Model.Selections.Ranges.ActiveRange.Right;
				int row = 0;
				int col = 0;
				bool flag = false;

				if(this.owner.Model.Selections.Ranges.Count > 0)
					flag = true;

				cc = this.owner.CurrentCell;
				Point p = this.owner.PointToClient(Control.MousePosition);
				this.owner.PointToRowCol(p, out row, out col);
				if(flag)
				{
					if(row > bottom || row < top || col < left || col > right)
					{
						ActiveXSnapshot.FakeLeftMouseClick(this.owner, p);
					}
					flag = false;
				}
				this.owner.CurrentCell.MoveTo(row,col,GridSetCurrentCellOptions.ScrollInView);
			}

			***/// end of non executed code
		}
		GridRangeInfo range;

		object val;
		private void GetDraggedRangesFilled()
		{
			if(range != null)
				for(int top = range.Top; top <= range.Bottom; top++)
					for(int left = range.Left; left <= range.Right; left++)
						this.owner.Model[top,left].CellValue = val;	

			// Get that selected too...
			this.owner.Selections.Add(GridRangeInfo.Cells(range.Top, range.Left, range.Bottom, range.Right));
			this.owner.CurrentCell.MoveTo(-1, -1); // does not affect the focus of the selected cells...
		}

		public void CancelMode()
		{
			owner.AutoScrolling = ScrollBars.None;
			dragWindow.Visible = false;
			dragWindow = null;
		}

		public int HitTest(MouseEventArgs e, IMouseController controller)
		{
			lastHitTestCode = GridHitTestContext.None;
			if (e.Button == MouseButtons.Left)
			{
				GridRangeInfo range = owner.Selections.Ranges.ActiveRange;
				if (!range.IsEmpty)
				{
					Point point = new Point(e.X, e.Y);
					point.Offset(-delta, -delta);
					int rowIndex, colIndex;
					owner.PointToRowCol(point, out rowIndex, out colIndex);

					if (rowIndex == range.Bottom && colIndex == range.Right)
					{
						point.Offset(delta*2+1, delta*2+1);
						int rowIndex2, colIndex2;
						owner.PointToRowCol(point, out rowIndex2, out colIndex2);
						if (rowIndex2 != rowIndex && colIndex2 != colIndex)
							lastHitTestCode = HitExcelMarker;
					}
				}
			}
			return lastHitTestCode;
		}
	}
}